/**
 * Created by ruckola on 10-01-17.
 * 
 * This extended command class is the build-in macro-command.
 * It depends on the user configuration file, which can be specified in the module's 'config.json'.
 * The user configuration must have a key called 'cmdSequence', in which a chain of commands
 * is specified as an array. Since those commands depend on each other, they need to be executed in
 * the right order. (start with index 0 and increment with 1)
 * IMPORTANT NOTE!: Each command that extends the abstract Command-class must return a promise.
 * Since each command has a build in CommandResult object, you can use this to store your data,
 * and pass it around. 
 */
var Command = require('./command');
/**
 * @class Macro
 * @augments Command
 * @classdesc This extended command class is the build-in macro-command. It's implementaion 
 * of the execute() method, is a mechanism to synchronously run subcommands and return there
 * individual results as an array of CommandResults, wrapped in a promise.   
 * It depends on the user configuration file, which can be specified in the module's 'config.json'.
 * The user configuration must have a key called 'cmdSequence', in which a chain of commands
 * is specified as an array. Since those commands are assumed to depend on each other, they
 * will be executed in order from left to right. (start with index 0 and increment with 1)
 * IMPORTANT NOTE!: Each command that extends the abstract Command-class must return a promise.
 * Since each command has a build in CommandResult object, you can use this to store your data,
 * and pass it around. 
 * @author ruckola
 */
class Macro extends Command {

    execute(context) {
        var self = this;
        return new Promise(function (resolve, reject) {
            var Helper = require('../core/helper');
            
            if (context.parameters.userConfiguration) {
                var pMacroCfg = Helper.getMacroConfiguration(
                   context.parameters.userConfiguration,
                    context.parameters.macroID);

                Promise.all([pMacroCfg]).then((values) => {

                    context.parameters.configuration = values[0];
                    var cfg = context.parameters.configuration;
                    var seq = cfg.cmdSequence;
                    var CommandContext = require('../core/commandContext');
                    var ctxt = new CommandContext();

                    seq.reduce((prevCmd, curCmd) => {
                        
                        ctxt.parameters.action = curCmd;
                        return prevCmd.then((resultObj) => {

                            if (resultObj)
                                self.result.data.push(resultObj);

                            //make sure to get the namespace right
                            //if namespace parameter in config available
                            //then use it.
                            //else resolve individual command names.
                            //In this situation, you can use different namespaces
                            //by using prefixes in the commandsequence.     
                            var cd = context.parameters.userCommandsRootDir; //commanddir
                            if (context.parameters.configuration.namespace) {
                                var path = require('path');
                                var cd = path.join(context.parameters.userCommandsRootDir,
                                    context.parameters.configuration.namespace)
                            }

                            var cmd = CommandFactory.getCommand(curCmd, cd);
                            // make the results of previous subcommands available
                            // via the commandContext.
                            ctxt.parameters.prevResults = self.result.data;
                            return cmd.execute(ctxt);
                        });
                    }, Promise.resolve()).then(
                        (resultObj) => {

                            //set the result of the macro command itself.
                            self.result.cmd = self.constructor.name;
                            self.result.data.push(resultObj);
                            self.result.msg = "The macro has finished successfully!";
                            resolve(self.result);
                        },
                        (e) => console.log(e)
                        );
                });

            }
        })
    }
}

module.exports = Macro;