/**
 * Created by ruckola on 9-12-16.
 *
 * These tests need a default command class in the ./bin/commands directory in order
 * to pass successfully.
 */
const assert = require('chai').assert;
const CommandFactory = require('../../bin/core/commandFactory');
const fs = require('fs');
const path = require('path');
describe("Test suite for (static) class CommandFactory", function () {

    it("The getCommand method should trow an error if parameter 'action' isn't of type string.", function () {

        function wrongActionType () {
            var action = {foo: 'bar'};
            var cmd = CommandFactory.getCommand(action);
            return cmd;
        }

        assert.throws(wrongActionType,Error);
    });

    it("The getCommand method should trow an error if parameter 'action' contains illegal characters.", function () {

        function wrongActionType () {
            var action = 'd efault';
            var cmd = CommandFactory.getCommand(action);
            return cmd;
        }

        assert.throws(wrongActionType,Error);
    });

    it("The parameter 'action' should be converted to lowercase.", function () {

            var action = 'DEFaulT';
            var cmd = CommandFactory.getCommand(action);

        assert.equal(cmd.props.className,'default');
    });

    it('The relative filePath should be like "../commands/{action}.js".', function () {

        var action = 'default';
        var cmd = CommandFactory.getCommand(action);

        assert.equal(cmd.props.filePath,path.normalize('../commands/default.js'));
    });

    it('The absolute path should be an existing directory', function () {

        var action = 'default';
        var cmd = CommandFactory.getCommand(action);

        assert.equal(fs.existsSync(cmd.props.absolutePath),true);
    });

    it("The getCommand method should trow an error if the absolutePath doesn't exist.", function () {

        function nonExistingAction () {
            var action = 'nonExistingCommand';
            var cmd = CommandFactory.getCommand(action);
            return cmd;
        }

        assert.throws(nonExistingAction,Error);
    });

    it("The getCommand method should return a object of type command.", function () {

        function checkCmdType () {
            var action = 'default';
            var cmd = CommandFactory.getCommand(action);
            return cmd;
        }
        var c = checkCmdType();

        assert.equal(c instanceof Command, true);
    });

    it("The getCommand method should return a object of type DefaultCommand.", function () {

        function checkCmdType () {
            var b = false;
            var action = 'default';
            var cmd = CommandFactory.getCommand(action);

            const DefaultCommand = require('../../bin/commands/default');
            var dc = new DefaultCommand;

            return(cmd.__proto__ === dc.__proto__);
        }

        assert.equal(checkCmdType(), true);
    });


})