var assert = require('chai').assert;
var should = require('chai').should;
var Command = require('../../bin/commands/command');
var DefaultCommand = require('../../bin/commands/default');
var CommandContext = require('../../bin/core/commandContext');

describe("Test suite for the abstract 'command' class", function(){

    it("The command class should throw an error if instantiated,\n       because it is abstract.",function () {

        function instantiate () {
            var cmd = new Command;
        }

        assert.throws(instantiate,Error);
    });

    it("The execute method should throw an error if no context is available", function () {
        function instantiate () {
            var cmd = new DefaultCommand;
            var result = cmd.execute();
            //console.log(result.msg);
        }
        assert.throws(instantiate,Error);

    });

    it("The execute method should throw an error if context is of the wrong type.", function () {
        function instantiate () {

            var paramOfWrongType = "justAString";
            var cmd = new DefaultCommand;
            var result = cmd.execute(paramOfWrongType);
            return result;

        }
        assert.throws(instantiate,Error);

    });

    it("The execute method should return a result object with a msg as string if context is of the correct type.", function () {
        function instantiate () {

            var ctxt = new CommandContext();
            var cmd = new DefaultCommand;
            var result = cmd.execute(ctxt);
            return result;
        }

        var result = instantiate();

        assert.equal(typeof result.msg,'string');
    });


});